"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleUpdateConnectionOptions = void 0;
const markable_form_fields_1 = require("../constants/markable-form-fields");
const check_for_invalid_character_in_host_1 = require("./check-for-invalid-character-in-host");
function handleUpdateConnectionOptions(action, { errors: initialErrors, connectionStringUrl, connectionOptions, connectionStringInvalidError, warnings, }) {
    const { key, value, currentTab } = action;
    if (currentTab === 'none') {
        return {
            connectionStringUrl,
            connectionOptions: {
                connectionString: connectionStringUrl.toString(),
            },
            errors: initialErrors,
            warnings,
            connectionStringInvalidError,
        };
    }
    const errors = validateSshOptions(currentTab, key, value, connectionStringUrl.isSRV, connectionOptions.sshTunnel);
    if (!connectionOptions.sshTunnel) {
        connectionOptions.sshTunnel = {};
    }
    const response = {
        connectionStringUrl,
        connectionOptions: {
            ...connectionOptions,
            sshTunnel: {
                ...connectionOptions.sshTunnel,
                [key]: value,
            },
            connectionString: connectionStringUrl.toString(),
        },
        errors: [
            ...initialErrors.filter(({ fieldName }) => fieldName !== markable_form_fields_1.MARKABLE_FORM_FIELD_NAMES.IS_SSH),
        ],
        warnings,
        connectionStringInvalidError,
    };
    if (Object.values(errors).length > 0) {
        response.errors.push({
            fieldName: markable_form_fields_1.MARKABLE_FORM_FIELD_NAMES.IS_SSH,
            errors,
        });
    }
    return response;
}
exports.handleUpdateConnectionOptions = handleUpdateConnectionOptions;
const validateSshOptions = (currentTab, key, value, isSRV, sshOptions) => {
    const errors = currentTab === 'identity'
        ? validateIdentityForm(key, value, isSRV, sshOptions)
        : validatePasswordOrSocksForm(key, value, isSRV, sshOptions);
    for (const key in errors) {
        if (!errors[key]) {
            delete errors[key];
        }
    }
    return errors;
};
const validatePasswordOrSocksForm = (key, value, isSRV, sshOptions) => {
    const errors = {};
    switch (key) {
        case 'host':
            errors['host'] = getHostValidationError(value, isSRV);
            break;
        case 'port':
            errors['port'] = getPortValidationError(value);
            break;
        case 'username': {
            if (!value && (sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.password)) {
                errors['username'] = 'Username is required.';
            }
            break;
        }
        case 'password': {
            if (value && !(sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.username)) {
                errors['username'] = 'Username is required.';
            }
            break;
        }
    }
    return errors;
};
const validateIdentityForm = (key, value, isSRV, sshOptions) => {
    const errors = {};
    switch (key) {
        case 'host':
            errors['host'] = getHostValidationError(value, isSRV);
            break;
        case 'port':
            errors['port'] = getPortValidationError(value);
            break;
        case 'username':
            if (!value && (sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.identityKeyFile)) {
                errors['username'] = 'Username is required.';
            }
            break;
        case 'identityKeyPassphrase':
            if (value && !(sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.identityKeyFile)) {
                if (!(sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.identityKeyFile)) {
                    errors['identityKeyFile'] = 'File is required along with passphrase.';
                }
                if (value && !(sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.username)) {
                    errors['username'] = 'Username is required.';
                }
            }
            break;
        case 'identityKeyFile':
            if (value && !(sshOptions === null || sshOptions === void 0 ? void 0 : sshOptions.username)) {
                errors['username'] = 'Username is required along with identity file.';
            }
            break;
    }
    return errors;
};
const getHostValidationError = (value, isSRV) => {
    if (!value) {
        return 'Host is required.';
    }
    try {
        (0, check_for_invalid_character_in_host_1.checkForInvalidCharacterInHost)(value, isSRV);
    }
    catch (e) {
        return e.message;
    }
};
const getPortValidationError = (value) => {
    if (value < 1) {
        return 'Port must be a valid port number.';
    }
};
//# sourceMappingURL=connection-options-handler.js.map